% Designing of P and PI controller based on ROOT LOCUS method and
% desired closed-loop overshoot (and settling time)for step changing in SP
clc
clear all
osd=0.2; %Desired closed-loop overshoot (P & PI controller)
tsd=10; %Desired closed-loop settling time (PI controller) 
action=+1; %Defining action of controller (-1 for Direct & +1 for Reverse)
s=tf('s'); %Defining "s" as a transfer function

%Open loop transfer function without controller
Gp=(2-s)/((s+1)*(s+2)^2);Gpr=exp(-s)/((s+1)*(s+2));
Gol=action*Gp;Golr=action*Gpr; 

%Designing of P controller
K=logspace(-1,.5);
R=(rlocus(Gol,K))'
%Branch of rlocus with positive Imag. part
br=input('What is the branch number? ');
OS=exp(pi*real(R(:,br))./(imag(R(:,br))+eps));
figure
plot(K,OS*100,'k')
xlabel('Controller Gain');
ylabel('Overshoot (%)');
kc1=interp1q(OS',K',osd) %Gain of proportional controller
Golp=Gol*kc1;Golpr=Golr*kc1; %Open loop TF with P controller
figure
subplot(2,2,1)
rlocus(Golp)
title('Root locus of closed-loop poles(P controller)') 
Gcl1=feedback(Golp,1);Gcl1r=feedback(Golpr,1); %Closed-loop TF

%Designing of PI controller
zc=-log(osd)/sqrt(pi^2+log(osd)^2); %Calculation of damping coefficient
tt=tsd*zc/4; %Calculation of time constant
s1=-zc/tt+1i*sqrt(1-zc^2)/tt; %Desired closed-loop poles
[num,den]=tfdata(Gol,'v');
Gol_s1=polyval(num,s1)/polyval(den,s1); %The value of Gol at s1
aGol_s1=angle(Gol_s1); %The angle of Gol at s1
teta=angle(s1);

%Angle of PI controller at desired closed-loop pole
kk=0;
for count=1:5
    phy=-(2*kk+1)*pi-aGol_s1;
    if phy>-teta && phy<0, break, end
    phy=-(-2*kk+1)*pi-aGol_s1;
    if phy>-teta && phy<0, break, end
    kk=kk+1;
end
if kk==5
    disp('The control objectives infeseable with a PI controller')
else
    alpha=phy+teta;
    delta=0;
    if alpha>pi/2, delta=1; end

    ti2=1/(imag(s1)/tan(alpha-pi*delta)-real(s1))
    kc2=1/(abs(Gol_s1)*abs(1+1/(ti2*s1)))
    %Open loop TF with PI controller
    Golpi=Gol*kc2*(1+1/(ti2*s));Golpir=Golr*kc2*(1+1/(ti2*s));
    subplot(2,2,2)
    rlocus(Golpi)
    title('Root locus of closed-loop poles(PI controller)') 
    Gcl2=feedback(Golpi,1);Gcl2r=feedback(Golpir,1); %Closed-loop TF
    subplot(2,2,3)
    pzmap(Gcl1);
    title('Closed-loop poles and zeros(P controller)') 
    subplot(2,2,4)
    pzmap(Gcl2)
    title('Closed-loop poles and zeros(PI controller)') 
    figure
    %Closed-loop response for a unit step change in SP
    [Y1,T1]=step(Gcl1,20);[Y1r,T1r]=step(Gcl1r,20);
    [Y2,T2]=step(Gcl2,20);[Y2r,T2r]=step(Gcl2r,20);
    y1max=max(Y1);y2max=max(Y2);
    n1=size(Y1);n2=size(Y2);
    y1f=Y1(n1(1));y2f=Y2(n2(1));
    os1=(y1max/y1f-1)*100 %Resulted overshoot with using P controller
    os2=(y2max/y2f-1)*100 %Resulted overshoot with using PI controller
    plot(T1,Y1,'k-',T2,Y2,'k--',T1r,Y1r,'k-.',T2r,Y2r,'k:','LineWidth',1.5)
    legend('P controller (a)','PI controller (a)',...
        'P controller (e)','PI controller (e)','Location','SouthEast')
    xlabel('Time(sec)')
    ylabel('Process output')
    title(['Closed-loop response for a unit step change in SP';...
           '        (a)proximate and (e)xact model           '])
    grid
end
